@@ -115,6 +115,97 @@ def delete(args):
                              getJSON(domain, args.infile, False))
 
 
+# Retrieve template
+def template_get(args):
+    import processing
+    password = getPassword(args)
+    t = token(args.username, password, args.server)
+    if args.template:
+        template = args.template
+        processing.getTemplate(args.server, t, template)
+    else:
+        processing.getAllTemplates(args.server, t)
+
+
+# Create or update template
+def template_create_or_update(args):
+    import processing
+    import converter
+    domain = args.domain
+    if args.template:
+        identifier = args.template
+    else:
+        identifier = domain.replace('.', '_')
+    o = converter.JSONConvert(domain)
+    dnsaddr = args.dnsaddr
+    desc = args.desc if args.desc else ''
+    password = getPassword(args)
+    t = token(args.username, password, args.server)
+    o.generateTemplate(domain, dnsaddr, desc=desc)
+    if args.template:
+        processing.updateTemplate(args.server, t, identifier, o.record)
+    else:
+        processing.createTemplate(args.server, t, identifier, o.record)
+
+
+# Delete template
+def template_delete(args):
+    import processing
+    import converter
+    password = getPassword(args)
+    if args.template:
+        template = args.template
+    t = token(args.username, password, args.server)
+    processing.deleteTemplate(args.server, t, template)
+
+
+def setoption(obj, keyword, prefix=False, required=False):
+    if keyword == 'server':
+        obj.add_argument(
+            '-s', dest='server', required=True,
+            help='specify TonicDNS Server hostname or IP address')
+    if keyword == 'username':
+        obj.add_argument('-u', dest='username', required=True,
+                         help='TonicDNS username')
+    if keyword == 'password':
+        group = obj.add_mutually_exclusive_group(required=True)
+        group.add_argument('-p', dest='password',
+                           help='TonicDNS password')
+        group.add_argument('-P', action='store_true',
+                           help='TonicDNS password prompt')
+    if keyword == 'infile':
+        obj.add_argument('infile', action='store',
+                         help='pre-converted text file')
+    if keyword == 'template':
+        msg = 'specify template identifier'
+        if prefix:
+            msg = prefix + msg
+        if required:
+            obj.add_argument('--template', action='store',
+                             required=True, help=msg)
+        else:
+            obj.add_argument('--template', action='store',
+                             help=msg)
+    if keyword == 'search':
+        obj.add_argument('--search', action='store',
+                         help='partial match search')
+
+
+def conn_options(obj, server=False, username=False, password=False):
+    if server and username and password:
+        obj.set_defaults(server=server, username=username,
+                         password=password)
+    elif server and username:
+        obj.set_defaults(server=server, username=username)
+
+    if not server:
+        setoption(obj, 'server')
+    if not username:
+        setoption(obj, 'username')
+    if not password:
+        setoption(obj, 'password')
+
+
 # Define sub-commands and command line options
 def parse_options():
     import argparse
@@ -127,104 +218,73 @@ def parse_options():
     if os.path.isfile(CONFIGFILE):
         server, username, password = checkConfig(CONFIGFILE)
 
-    parser = argparse.ArgumentParser(description='usage')
-    parser.add_argument('-v', '--version', action='version',
+    prs = argparse.ArgumentParser(description='usage')
+    prs.add_argument('-v', '--version', action='version',
                         version=__version__)
-    parser.add_argument('--config', action='store',
+    prs.add_argument('--config', action='store',
                         help='config file path')
 
-    subparsers = parser.add_subparsers(help='commands')
+    subprs = prs.add_subparsers(help='commands')
 
     # Convert and print JSON
-    parser_show = subparsers.add_parser('show',
-                                         help='show converted JSON')
-    parser_show.add_argument('infile', action='store',
-                               help='pre-converted text file')
-    parser_show.set_defaults(func=show)
+    prs_show = subprs.add_parser('show',
+                                    help='show converted JSON')
+    setoption(prs_show, 'infile')
+    prs_show.set_defaults(func=show)
 
     # Retrieve records
-    parser_get = subparsers.add_parser(
+    prs_get = subprs.add_parser(
         'get', help='retrieve records of specific zone')
-    if server and username and password:
-        parser_get.set_defaults(server=server, username=username,
-                                password=password)
-    elif server and username:
-        parser_get.set_defaults(server=server, username=username)
-
-    parser_get.add_argument('--domain', action='store',
+    prs_get.add_argument('--domain', action='store',
                             help='specify domain FQDN')
-    if not server:
-        parser_get.add_argument(
-            '-s', dest='server', required=True,
-            help='specify TonicDNS Server hostname or IP address')
-    if not username:
-        parser_get.add_argument('-u', dest='username', required=True,
-                                help='TonicDNS username')
-    if not password:
-        group_get = parser_get.add_mutually_exclusive_group(required=True)
-        group_get.add_argument('-p', dest='password',
-                               help='TonicDNS password')
-        group_get.add_argument('-P', action='store_true',
-                               help='TonicDNS password prompt')
-    parser_get.add_argument('--search', action='store',
-                            help='partial match search')
-    parser_get.set_defaults(func=get)
+    conn_options(prs_get, server, username, password)
+    setoption(prs_get, 'search')
+    prs_get.set_defaults(func=get)
 
     # Create records
-    parser_create = subparsers.add_parser(
+    prs_create = subprs.add_parser(
         'create', help='create records of specific zone')
-
-    if server and username and password:
-        parser_create.set_defaults(server=server, username=username,
-                                password=password)
-    elif server and username:
-        parser_create.set_defaults(server=server, username=username)
-
-    parser_create.add_argument('infile', action='store',
-                                 help='pre-converted text file')
-    if not server:
-        parser_create.add_argument('-s', dest='server', required=True,
-                                   help='specify TonicDNS hostname|IP address')
-    if not username:
-        parser_create.add_argument('-u', dest='username', required=True,
-                                   help='TonicDNS username')
-    if not password:
-        group_create = \
-            parser_create.add_mutually_exclusive_group(required=True)
-        group_create.add_argument('-p', dest='password',
-                               help='TonicDNS password')
-        group_create.add_argument('-P', action='store_true',
-                               help='TonicDNS password prompt')
-    parser_create.set_defaults(func=create)
+    setoption(prs_create, 'infile')
+    conn_options(prs_create, server, username, password)
+    prs_create.set_defaults(func=create)
 
     # Delete records
-    parser_delete = subparsers.add_parser(
+    prs_delete = subprs.add_parser(
         'delete', help='delete records of specific zone')
+    setoption(prs_delete, 'infile')
+    conn_options(prs_delete, server, username, password)
+    prs_delete.set_defaults(func=delete)
 
-    if server and username and password:
-        parser_delete.set_defaults(server=server, username=username,
-                                password=password)
-    elif server and username:
-        parser_delete.set_defaults(server=server, username=username)
+    # Retrieve template
+    prs_tmpl_get = subprs.add_parser(
+        'tmpl_get', help='retrieve templates')
+    setoption(prs_tmpl_get, 'template')
+    conn_options(prs_tmpl_get, server, username, password)
+    prs_tmpl_get.set_defaults(func=template_get)
 
-    parser_delete.add_argument('infile', action='store',
-                                 help='pre-converted text file')
-    if not server:
-        parser_delete.add_argument('-s', dest='server', required=True,
-                                   help='specify TonicDNS hostname|IP address')
-    if not username:
-        parser_delete.add_argument('-u', dest='username', required=True,
-                                   help='TonicDNS username')
-    if not password:
-        group_delete = \
-            parser_delete.add_mutually_exclusive_group(required=True)
-        group_delete.add_argument('-p', dest='password',
-                                  help='TonicDNS password')
-        group_delete.add_argument('-P', action='store_true',
-                                  help='TonicDNS password prompt')
-    parser_delete.set_defaults(func=delete)
+    # create or update template
+    prs_tmpl_create_update = subprs.add_parser(
+        'tmpl_create_update', help='create or update template')
+    prs_tmpl_create_update.add_argument(
+        '--domain', action='store', required=True,
+        help='create template with specify domain')
+    setoption(prs_tmpl_create_update, 'template', 'update template with ')
+    prs_tmpl_create_update.add_argument(
+        '--dnsaddr', action='store', required=True,
+        help='specify IP address of NS record')
+    prs_tmpl_create_update.add_argument(
+        '--desc', action='store', help='description')
+    conn_options(prs_tmpl_create_update, server, username, password)
+    prs_tmpl_create_update.set_defaults(func=template_create_or_update)
 
-    args = parser.parse_args()
+    # delete template
+    prs_tmpl_delete = subprs.add_parser(
+        'tmpl_delete', help='delete template')
+    setoption(prs_tmpl_delete, 'template', required=True)
+    conn_options(prs_tmpl_delete, server, username, password)
+    prs_tmpl_delete.set_defaults(func=template_delete)
+
+    args = prs.parse_args()
     return args
 
 
